from zope.interface import implements, Interface

from Products.Five import BrowserView

from collective.ots import otsMessageFactory as _
from collective.ots.utils import summarize
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class ISummarizeView(Interface):
    """
    Summarize view interface
    """


class SummarizeView(BrowserView):
    """
    Summarize browser view
    """
    implements(ISummarizeView)
    template = ViewPageTemplateFile('summarizeview.pt')

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_summary(self):
        summary = ''
        if hasattr(self.context, 'Description') and hasattr(self.context, 'SearchableText'):
            text = self.context.SearchableText().strip().lstrip(self.context.id).lstrip().lstrip(self.context.Title())
            summary = summarize(text, self.context.Language()).strip()
        return summary

    def __call__(self):
        form = self.request.form
        if form.has_key('form.button.save'):

            desc = form.get('description', '')
            self.context.setDescription(desc)
            self.request.response.redirect(self.context.absolute_url() + '/edit')
            return ''
        elif form.has_key('form.button.cancel'):
            self.request.response.redirect(self.context.absolute_url() + '/view')
            return ''
        return self.template()



# ots integrattion
import logging
try:
    import ots
    LIB_OTS = True
except ImportError:
    LIB_OTS = False

from subprocess import call
from tempfile import NamedTemporaryFile
from Products.CMFCore.utils import getToolByName

logger = logging.getLogger('collective.ots')

LANGUAGES = ['bg', 'ca', 'cs', 'cy', 'da', 'de', 'el', 'en', 'eo', 'es',
        'et', 'eu', 'fi', 'fr', 'ga', 'gl', 'he', 'hu', 'ia', 'id', 'is',
        'it', 'lv', 'mi', 'ms', 'mt', 'nl', 'nn', 'pl', 'pt', 'ro', 'ru',
        'sv', 'tl', 'tr', 'uk', 'yi']

def summarize(text, language='en', words=70):
    """ Summarize the given text in language to a maximum of words"""
    tmp = NamedTemporaryFile()
    tmp.write(text)
    tmp.flush()
    txt_len = len(text.split())
    if txt_len == 0:
        return ''
    ratio = max([int(float(words)/float(txt_len) * 100), 2])
    ratio = min(ratio, 20)
    if LIB_OTS:

        if language in LANGUAGES:
            o = ots.OTS(language)
        else:
            o = ots.OTS()
        o.parse(tmp.name, ratio)
        tmp.close()
        return o.asText()
    else:
        outfile = NamedTemporaryFile()
        args =['ots']
        args.append('--ratio=%i' % ratio)
        if language in LANGUAGES:
            args.append('--dic=%s' % language)
        args.append('--out=%s' % outfile.name)
        args.append(tmp.name)
        try:
            if call(args) == 0:
                outfile.seek(0)
                result = outfile.read()
            else:
                result =''
        except OSError:
            logger.info("OTS not found")
            result = ''
        tmp.close()
        outfile.close()
        return result

def generate_description_searchable_text(context, event):
    if hasattr(context, 'Description') and hasattr(context, 'SearchableText'):
        if context.Description() == '':
            text = context.SearchableText().strip().lstrip(context.id).lstrip().lstrip(context.Title())
            summary = summarize(text, context.Language()).strip()
            context.setDescription(summary)





import unittest
import doctest

#from zope.testing import doctestunit
#from zope.component import testing, eventtesting

from Testing import ZopeTestCase as ztc

from collective.ots.tests import base


def test_suite():
    return unittest.TestSuite([

        # Demonstrate the main content types
        ztc.ZopeDocFileSuite(
            'README.txt', package='collective.ots',
            test_class=base.FunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE |
                doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),

        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
"""
This module contains the tool of collective.ots
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.3'

long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' +
    read('CHANGES.txt')
#    + '\n' +
#    'Detailed Documentation\n'
#    '**********************\n'
#    + '\n' +
#    read('collective', 'ots', 'README.txt')
#    + '\n' +
#    'Contributors\n'
#    '************\n'
#    + '\n' +
#    read('CONTRIBUTORS.txt')
    + '\n' +
    'Download\n'
    '********\n')

tests_require = ['zope.testing']

setup(name='collective.ots',
      version=version,
      description="""Writing a good summary (description) is hard,
                let the Open Text Summarizer (OTS) do the heavy lifting.
                This tool extracts a summary of the content and sets it
                as the description of the content item you added""",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='',
      author='Christian Ledermann',
      author_email='christian.ledermann@gmail.com',
      url='http://plone.org/products/collective.ots',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        # -*- Extra requirements: -*-
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite='collective.ots.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      #setup_requires=["PasteScript"],
      #paster_plugins=["ZopeSkel"],
      )
